@@ -31,10 +31,14 @@
 
 #include <gtk/gtk.h>
 #include <clutter/clutter.h>
-#include <clutter-gtk/gtk-clutter-embed.h>
+#include <clutter-gtk/clutter-gtk.h>
 
 #include "gtk-champlain-embed.h"
 
+#if (GTK_MAJOR_VERSION == 2 && GTK_MINOR_VERSION <= 12)
+#define gtk_widget_get_window(widget) ((widget)->window)
+#endif
+
 enum
 {
   /* normal signals */
