@@ -65,7 +65,6 @@
 #include "champlain-zoom-level.h"
 
 #include <clutter/clutter.h>
-#include <clutter-cairo/clutter-cairo.h>
 #include <glib.h>
 #include <glib-object.h>
 #include <math.h>
@@ -135,11 +134,6 @@ struct _ChamplainViewPrivate
   gdouble longitude;
   gdouble latitude;
 
-  /* Hack to get smaller x,y coordinates as the clutter limit is G_MAXINT16 */
-  ChamplainIntPoint anchor;
-  gdouble anchor_zoom_level; /* the zoom_level for which the current anchor has
-                                been computed for */
-
   Map *map; /* Contains the current map model */
 
   ClutterActor *finger_scroll; /* Contains the viewport */
@@ -203,10 +197,7 @@ static gboolean finger_scroll_button_press_cb (ClutterActor *actor,
 static void update_license (ChamplainView *view);
 static void license_set_position (ChamplainView *view);
 static void view_load_visible_tiles (ChamplainView *view);
-static void view_position_tile (ChamplainView* view, ChamplainTile* tile);
-static void view_tiles_reposition (ChamplainView* view);
 static void view_update_state (ChamplainView *view);
-static void view_update_anchor (ChamplainView *view, gint x, gint y);
 static gboolean view_set_zoom_level_at (ChamplainView *view,
     gint zoom_level,
     gint x,
@@ -232,7 +223,7 @@ viewport_get_current_longitude (ChamplainViewPrivate *priv)
   if (!priv->map)
     return 0.0;
 
-  return viewport_get_longitude_at (priv, priv->anchor.x +
+  return viewport_get_longitude_at (priv,
       priv->viewport_size.x + priv->viewport_size.width / 2.0);
 }
 
@@ -253,8 +244,7 @@ viewport_get_current_latitude (ChamplainViewPrivate *priv)
     return 0.0;
 
   return viewport_get_latitude_at (priv,
-      priv->anchor.y + priv->viewport_size.y +
-      priv->viewport_size.height / 2.0);
+      priv->viewport_size.y + priv->viewport_size.height / 2.0);
 }
 
 static gboolean
@@ -288,9 +278,7 @@ marker_reposition_cb (ChamplainMarker *marker,
       x = champlain_map_source_get_x (priv->map_source, priv->zoom_level, marker_priv->lon);
       y = champlain_map_source_get_y (priv->map_source, priv->zoom_level, marker_priv->lat);
 
-      clutter_actor_set_position (CLUTTER_ACTOR (marker),
-        x - priv->anchor.x,
-        y - priv->anchor.y);
+      clutter_actor_set_position (CLUTTER_ACTOR (marker), x, y);
     }
 }
 
@@ -351,7 +339,6 @@ create_initial_map (ChamplainView *view)
   clutter_container_add_actor (CLUTTER_CONTAINER (priv->map_layer), group);
 
   g_idle_add (marker_reposition, view);
-  view_tiles_reposition (view);
   update_license (view);
 
   g_object_notify (G_OBJECT (view), "zoom-level");
@@ -362,7 +349,7 @@ static void
 license_set_position (ChamplainView *view)
 {
   ChamplainViewPrivate *priv = view->priv;
-  guint width, height;
+  gfloat width, height;
 
   if (!priv->license_actor)
     return;
@@ -381,7 +368,7 @@ draw_polygon (ChamplainView *view, ChamplainPolygon *polygon)
   if (polygon->priv->visible == FALSE)
     return;
 
-  cr = clutter_cairo_create (CLUTTER_CAIRO (polygon->priv->actor));
+  cr = clutter_cairo_texture_create (CLUTTER_CAIRO_TEXTURE (polygon->priv->actor));
 
   /* Clear the drawing area */
   cairo_set_operator (cr, CAIRO_OPERATOR_CLEAR);
@@ -402,8 +389,8 @@ draw_polygon (ChamplainView *view, ChamplainPolygon *polygon)
       y = champlain_map_source_get_y (priv->map_source, priv->zoom_level,
           point->lat);
 
-      x -= priv->viewport_size.x + priv->anchor.x;
-      y -= priv->viewport_size.y + priv->anchor.y;
+      x -= priv->viewport_size.x;
+      y -= priv->viewport_size.y;
 
       cairo_line_to (cr, x, y);
       list = list->next;
@@ -480,33 +467,17 @@ resize_viewport (ChamplainView *view)
   tidy_scrollable_get_adjustments (TIDY_SCROLLABLE (priv->viewport), &hadjust,
       &vadjust);
 
-  if (priv->zoom_level < 8)
-    {
-      lower = -priv->viewport_size.width / 2.0;
-      upper = champlain_zoom_level_get_width (priv->map->current_level) *
-          champlain_map_source_get_tile_size (priv->map_source) -
-          priv->viewport_size.width / 2.0;
-    }
-  else
-    {
-      lower = 0;
-      upper = G_MAXINT16;
-    }
+  lower = -priv->viewport_size.width / 2.0;
+  upper = champlain_zoom_level_get_width (priv->map->current_level) *
+      champlain_map_source_get_tile_size (priv->map_source) -
+      priv->viewport_size.width / 2.0;
   g_object_set (hadjust, "lower", lower, "upper", upper,
       "page-size", 1.0, "step-increment", 1.0, "elastic", TRUE, NULL);
 
-  if (priv->zoom_level < 8)
-    {
-      lower = -priv->viewport_size.height / 2.0;
-      upper = champlain_zoom_level_get_height (priv->map->current_level) *
-          champlain_map_source_get_tile_size (priv->map_source) -
-          priv->viewport_size.height / 2.0;
-    }
-  else
-    {
-      lower = 0;
-      upper = G_MAXINT16;
-    }
+  lower = -priv->viewport_size.height / 2.0;
+  upper = champlain_zoom_level_get_height (priv->map->current_level) *
+      champlain_map_source_get_tile_size (priv->map_source) -
+      priv->viewport_size.height / 2.0;
   g_object_set (vadjust, "lower", lower, "upper", upper,
       "page-size", 1.0, "step-increment", 1.0, "elastic", TRUE, NULL);
 
@@ -529,7 +500,7 @@ resize_viewport (ChamplainView *view)
               polygon->priv->actor);
         }
 
-      polygon->priv->actor = g_object_ref (clutter_cairo_new (
+      polygon->priv->actor = g_object_ref (clutter_cairo_texture_new (
           view->priv->viewport_size.width,
           view->priv->viewport_size.height));
       g_object_set (G_OBJECT (polygon->priv->actor), "visible",
@@ -981,9 +952,6 @@ champlain_view_init (ChamplainView *view)
   priv->viewport_size.y = 0;
   priv->viewport_size.width = 0;
   priv->viewport_size.height = 0;
-  priv->anchor.x = 0;
-  priv->anchor.y = 0;
-  priv->anchor_zoom_level = 0;
   priv->state = CHAMPLAIN_STATE_INIT;
   priv->latitude = 0.0f;
   priv->longitude = 0.0f;
@@ -1055,8 +1023,7 @@ viewport_x_changed_cb (GObject *gobject,
 {
   ChamplainViewPrivate *priv = view->priv;
 
-  ChamplainIntPoint rect;
-  ChamplainIntPoint old_anchor;
+  ChamplainFloatPoint rect;
 
   tidy_viewport_get_origin (TIDY_VIEWPORT (priv->viewport), &rect.x, &rect.y,
       NULL);
@@ -1065,31 +1032,10 @@ viewport_x_changed_cb (GObject *gobject,
       rect.y == priv->viewport_size.y)
       return;
 
-  old_anchor.x = priv->anchor.x;
-  old_anchor.y = priv->anchor.y;
-
-  view_update_anchor (view,
-      rect.x + priv->anchor.x + priv->viewport_size.width / 2.0,
-      rect.y + priv->anchor.y + priv->viewport_size.height / 2.0);
-
-  if (priv->anchor.x - old_anchor.x != 0)
-    {
-      ChamplainIntPoint diff;
-
-      diff.x = priv->anchor.x - old_anchor.x;
-      diff.y = priv->anchor.y - old_anchor.y;
-
-      DEBUG("Relocating the viewport by %d, %d", diff.x, diff.y);
-      tidy_viewport_set_origin (TIDY_VIEWPORT (priv->viewport),
-          rect.x - diff.x, rect.y - diff.y, 0);
-      return;
-    }
-
   priv->viewport_size.x = rect.x;
   priv->viewport_size.y = rect.y;
 
   view_load_visible_tiles (view);
-  view_tiles_reposition (view);
   marker_reposition (view);
   view_update_polygons (view);
 
@@ -1137,7 +1083,7 @@ update_license (ChamplainView *view)
 
   if (priv->show_license)
     {
-      priv->license_actor = g_object_ref (clutter_label_new_with_text ("sans 8",
+      priv->license_actor = g_object_ref (clutter_text_new_with_text ("sans 8",
           champlain_map_source_get_license (priv->map_source)));
       clutter_actor_set_opacity (priv->license_actor, 128);
       clutter_actor_show (priv->license_actor);
@@ -1175,55 +1121,6 @@ champlain_view_new (void)
   return g_object_new (CHAMPLAIN_TYPE_VIEW, NULL);
 }
 
-static void
-view_update_anchor (ChamplainView *view,
-    gint x,
-    gint y)
-{
-  ChamplainViewPrivate *priv = view->priv;
-  gboolean need_anchor = FALSE;
-  gboolean need_update = FALSE;
-
-  if (priv->zoom_level >= 8)
-    need_anchor = TRUE;
-
-  if (priv->anchor_zoom_level != priv->zoom_level ||
-      x - priv->anchor.x + priv->viewport_size.width >= G_MAXINT16 ||
-      y - priv->anchor.y + priv->viewport_size.height >= G_MAXINT16)
-    need_update = TRUE;
-
-  if (need_anchor && need_update)
-    {
-      gdouble max;
-
-      priv->anchor.x = x - G_MAXINT16 / 2;
-      priv->anchor.y = y - G_MAXINT16 / 2;
-
-      if ( priv->anchor.x < 0 )
-        priv->anchor.x = 0;
-      if ( priv->anchor.y < 0 )
-        priv->anchor.y = 0;
-
-      max = champlain_zoom_level_get_width (priv->map->current_level) *
-          champlain_map_source_get_tile_size (priv->map_source) -
-          (G_MAXINT16 / 2);
-      if (priv->anchor.x > max)
-        priv->anchor.x = max;
-      if (priv->anchor.y > max)
-        priv->anchor.y = max;
-
-      priv->anchor_zoom_level = priv->zoom_level;
-    }
-
-  if (need_anchor == FALSE)
-    {
-      priv->anchor.x = 0;
-      priv->anchor.y = 0;
-      priv->anchor_zoom_level = priv->zoom_level;
-    }
-  DEBUG ("New Anchor (%d, %d) for (%d, %d)", priv->anchor.x, priv->anchor.y, x, y);
-}
-
 /**
  * champlain_view_center_on:
  * @view: a #ChamplainView
@@ -1255,11 +1152,6 @@ champlain_view_center_on (ChamplainView *view,
   x = champlain_map_source_get_x (priv->map_source, priv->zoom_level, longitude);
   y = champlain_map_source_get_y (priv->map_source, priv->zoom_level, latitude);
 
-  view_update_anchor (view, x, y);
-
-  x -= priv->anchor.x;
-  y -= priv->anchor.y;
-
   tidy_viewport_set_origin (TIDY_VIEWPORT (priv->viewport),
     x - priv->viewport_size.width / 2.0,
     y - priv->viewport_size.height / 2.0,
@@ -1269,7 +1161,6 @@ champlain_view_center_on (ChamplainView *view,
   g_object_notify (G_OBJECT (view), "latitude");
 
   view_load_visible_tiles (view);
-  view_tiles_reposition (view);
   view_update_polygons (view);
   marker_reposition (view);
 }
@@ -1283,7 +1174,7 @@ timeline_new_frame (ClutterTimeline *timeline,
   gdouble lat;
   gdouble lon;
 
-  alpha = (double) clutter_alpha_get_alpha (ctx->alpha) / CLUTTER_ALPHA_MAX_ALPHA;
+  alpha = clutter_alpha_get_alpha (ctx->alpha);
   lat = ctx->to_latitude - ctx->from_latitude;
   lon = ctx->to_longitude - ctx->from_longitude;
 
@@ -1373,9 +1264,8 @@ champlain_view_go_to (ChamplainView *view,
    * is higher and if the points are far away
    */
   duration = 500 * priv->zoom_level / 2.0;
-  ctx->timeline = clutter_timeline_new_for_duration (duration);
-  ctx->alpha = clutter_alpha_new_full (ctx->timeline, CLUTTER_ALPHA_SINE_INC, NULL,
-      NULL);
+  ctx->timeline = clutter_timeline_new (duration);
+  ctx->alpha = clutter_alpha_new_full (ctx->timeline, CLUTTER_EASE_IN_OUT_CIRC);
 
   g_signal_connect (ctx->timeline, "new-frame", G_CALLBACK (timeline_new_frame),
       ctx);
@@ -1639,8 +1529,8 @@ gboolean champlain_view_get_coords_at (ChamplainView *view,
 {
   g_return_val_if_fail (CHAMPLAIN_IS_VIEW (view), FALSE);
   ChamplainViewPrivate *priv = view->priv;
-  gint actor_x, actor_y;
-  gint rel_x, rel_y;
+  gfloat actor_x, actor_y;
+  gdouble rel_x, rel_y;
 
   clutter_actor_get_transformed_position (priv->finger_scroll, &actor_x, &actor_y);
 
@@ -1649,10 +1539,10 @@ gboolean champlain_view_get_coords_at (ChamplainView *view,
 
   if (latitude)
     *latitude = viewport_get_latitude_at (priv,
-        priv->viewport_size.y + rel_y + priv->anchor.y);
+        priv->viewport_size.y + rel_y);
   if (longitude)
     *longitude = viewport_get_longitude_at (priv,
-        priv->viewport_size.x + rel_x + priv->anchor.x);
+        priv->viewport_size.x + rel_x);
 
   return TRUE;
 }
@@ -1665,9 +1555,6 @@ view_load_visible_tiles (ChamplainView *view)
   gint size;
   ChamplainZoomLevel *level;
 
-  viewport.x += priv->anchor.x;
-  viewport.y += priv->anchor.y;
-
   size = champlain_map_source_get_tile_size (priv->map_source);
   level = priv->map->current_level;
 
@@ -1751,7 +1638,8 @@ view_load_visible_tiles (ChamplainView *view)
             {
               DEBUG ("Loading tile %d, %d, %d", champlain_zoom_level_get_zoom_level (level), i, j);
               ChamplainTile *tile = champlain_tile_new ();
-              g_object_set (G_OBJECT (tile), "x", i, "y", j, "zoom-level", champlain_zoom_level_get_zoom_level (level), NULL);
+              g_object_set (G_OBJECT (tile), "x", i, "y", j, "zoom-level",
+                  champlain_zoom_level_get_zoom_level (level), NULL);
 
               g_signal_connect (tile, "notify::state", G_CALLBACK (tile_state_notify), view);
               clutter_container_add (CLUTTER_CONTAINER (champlain_zoom_level_get_actor (level)),
@@ -1767,51 +1655,11 @@ view_load_visible_tiles (ChamplainView *view)
   view_update_state (view);
 }
 
-static void
-view_position_tile (ChamplainView* view,
-    ChamplainTile* tile)
-{
-  ChamplainViewPrivate *priv = view->priv;
-
-  ClutterActor *actor;
-  gint x;
-  gint y;
-  guint size;
-
-  actor = champlain_tile_get_actor (tile);
-  x = champlain_tile_get_x (tile);
-  y = champlain_tile_get_y (tile);
-  size = champlain_tile_get_size (tile);
-
-  clutter_actor_set_position (actor,
-    (x * size) - priv->anchor.x,
-    (y * size) - priv->anchor.y);
-}
-
-static void
-view_tiles_reposition (ChamplainView* view)
-{
-  ChamplainViewPrivate *priv = view->priv;
-  gint i;
-
-  for (i = 0; i < champlain_zoom_level_tile_count (priv->map->current_level); i++)
-    {
-      ChamplainTile *tile = champlain_zoom_level_get_nth_tile (priv->map->current_level, i);
-
-      if (tile == NULL)
-        continue;
-
-      if (champlain_tile_get_state (tile) == CHAMPLAIN_STATE_DONE)
-        view_position_tile (view, tile);
-    }
-}
-
 static void
 tile_state_notify (GObject *gobject,
     GParamSpec *pspec,
     gpointer data)
 {
-  view_position_tile (CHAMPLAIN_VIEW (data), CHAMPLAIN_TILE (gobject));
   view_update_state (CHAMPLAIN_VIEW (data));
 }
 
@@ -1899,7 +1747,6 @@ champlain_view_set_map_source (ChamplainView *view,
 
   update_license (view);
   g_idle_add (marker_reposition, view);
-  view_tiles_reposition (view);
   champlain_view_center_on (view, priv->latitude, priv->longitude);
 }
 
@@ -2161,9 +2008,9 @@ view_set_zoom_level_at (ChamplainView *view,
   ClutterActor *group, *new_group;
   gdouble lon, lat;
   gint x_diff, y_diff;
-  gint actor_x, actor_y;
-  gint rel_x, rel_y;
-  gint x2, y2;
+  gfloat actor_x, actor_y;
+  gdouble rel_x, rel_y;
+  gfloat x2, y2;
   gdouble lat2, lon2;
 
   if (zoom_level == priv->zoom_level || ZOOM_LEVEL_OUT_OF_RANGE(priv, zoom_level))
@@ -2178,9 +2025,9 @@ view_set_zoom_level_at (ChamplainView *view,
 
   /* Keep the lon, lat where the mouse is */
   lon = viewport_get_longitude_at (priv,
-    priv->viewport_size.x + rel_x + priv->anchor.x);
+    priv->viewport_size.x + rel_x);
   lat = viewport_get_latitude_at (priv,
-    priv->viewport_size.y + rel_y + priv->anchor.y);
+    priv->viewport_size.y + rel_y);
 
   /* How far was it from the center of the viewport (in px) */
   x_diff = priv->viewport_size.width / 2 - rel_x;
@@ -2416,7 +2263,7 @@ champlain_view_add_polygon (ChamplainView *view,
   g_return_if_fail (CHAMPLAIN_IS_VIEW (view));
   g_return_if_fail (CHAMPLAIN_IS_POLYGON (polygon));
 
-  view->priv->polygons = g_list_append (view->priv->polygons, g_object_ref (polygon));
+  view->priv->polygons = g_list_append (view->priv->polygons, g_object_ref_sink (polygon));
 
   g_signal_connect (polygon, "notify",
       G_CALLBACK (notify_polygon_cb), view);
@@ -2428,7 +2275,7 @@ champlain_view_add_polygon (ChamplainView *view,
     return;
   }
 
-  polygon->priv->actor = g_object_ref (clutter_cairo_new (
+  polygon->priv->actor = g_object_ref (clutter_cairo_texture_new (
       view->priv->viewport_size.width,
       view->priv->viewport_size.height));
   g_object_set (G_OBJECT (polygon->priv->actor), "visible",
